@@ -34,7 +34,7 @@ except ImportError:
     HAS_PYFLAKES = False
 
 try:
-    from jslint import jslint
+    import pyjslint
     HAS_JSLINT = True
 except ImportError:
     HAS_JSLINT = False
@@ -171,7 +171,7 @@ def pyflakeshook(ui, repo, hooktype, node, pending, **kwargs):
 
 
 def jslint_check(data, filename):
-    analizing = jslint.check_JSLint(data)
+    analizing = pyjslint.check_JSLint(data)
     if len(analizing) != 1:
         print >> sys.stderr, "%s: problem decoding source" % (filename, )
         print >> sys.stderr, '\n'.join(analizing[1:])
