@@ -17,7 +17,6 @@
 
 import datetime
 import re
-import time
 
 from pkg_resources import iter_entry_points
 
@@ -66,10 +65,8 @@ def default_token_commands():
         }
 
 
-def hgtime2datetime(hgdate):
-    hgtime, hgtz = hgdate
-    dateinfo = list(time.gmtime(float(hgtime) - hgtz)[:-2]) + [utc]
-    return datetime.datetime(*dateinfo)
+def utc_time():
+    return datetime.datetime.utcnow().replace(tzinfo=utc)
 
 
 def load_ticket_commands():
@@ -216,31 +213,36 @@ class TicketUpdater(object):
             db.commit()
             tnotify.notify(ticket, newticket=0, modtime=date)
 
-    def compose_msg(self, changectx):
-        styles = {
-            'number': changectx.rev(),
-            'long-hex': changectx.hex()
-            }
-        styles['short-hex'] = styles['long-hex'][:12]
+    def compose_msg(self, changes):
+        msg = []
+        for changectx in changes:
+            styles = {
+                'number': changectx.rev(),
+                'long-hex': changectx.hex()
+                }
+            styles['short-hex'] = styles['long-hex'][:12]
 
-        try:
-            changeset = styles[self.changeset_style]
-        except KeyError:
-            self.ui.warn('Unsupported changeset style: "%s". Currently '
-                         'supported are: number, short-hex and long-hex.'
-                         'Using default (short-hex)' % self.changeset_style)
-            changeset = styles['short-hex']
+            try:
+                changeset = styles[self.changeset_style]
+            except KeyError:
+                self.ui.warn('Unsupported changeset style: "%s". Currently '
+                             'supported are: number, short-hex and long-hex.'
+                             'Using default (short-hex)'
+                             % self.changeset_style)
+                changeset = styles['short-hex']
 
-        if self.repo_name is not None:
-            changeset += u'/' + self.repo_name
+            if self.repo_name is not None:
+                changeset += u'/' + self.repo_name
 
-        return self.msg_template % {
-            'changeset': changeset,
-            'msg': unicode(changectx.description(), 'utf-8'),
-            }
+            msg.append(self.msg_template % {
+                    'changeset': changeset,
+                    'msg': unicode(changectx.description(), 'utf-8'),
+                    })
 
-    def __call__(self, changectx):
-        msg = self.compose_msg(changectx)
+        return u'\n'.join(msg)
+
+    def __call__(self, changes):
+        msg = self.compose_msg(changes)
 
         tickets = get_ticket_commands(msg)
 
@@ -248,8 +250,9 @@ class TicketUpdater(object):
 
         processed_tickets = self.execute_ticket_commands(tickets, env)
         self.execute_token_commands(msg, processed_tickets)
-        author = unicode(changectx.user(), 'utf-8')
-        date = hgtime2datetime(changectx.date())
+
+        author = unicode(changes[0].user(), 'utf-8')
+        date = utc_time()
         self.save_and_notify(processed_tickets, env, author, msg, date)
 
 
@@ -266,6 +269,14 @@ def ticket_updater(ui, repo, hooktype, node, source, url):
                              '(At [%(changeset)s]) %(msg)s')
     msg_template = unicode(msg_template, 'utf-8')
 
+    if hooktype == 'changegroup':
+        # fetch all remaining changesets
+        changes = [repo[rev]
+                   for rev in range(repo[node].rev(), len(repo.changelog))]
+    else:
+        # use just the current revision
+        changes = [repo[node]]
+
     ticket_updater = TicketUpdater(trac_env, repo_name, changeset_style,
                                    msg_template, ui)
-    ticket_updater(repo[node])
+    ticket_updater(changes)
